/*** complex.cpp  $Revision$  $Date$
*
*   Complex math routines
*
***/
#include <stdio.h>
#define _USE_MATH_DEFINES
#include <math.h>
#include "complex.h"

const double Complex::dgrad = 180.0/M_PI;

Complex operator *(Complex const &x, Complex const &y)    // Complex multiplication
{
	Complex z;
	z.re = x.re * y.re - x.im * y.im;
	z.im = x.re * y.im + x.im * y.re;
	return z;
}

Complex operator /(Complex const &x, Complex const &y)    // Complex division 
{
	Complex z;
	double denom;
	denom = y.re * y.re + y.im *y.im;
	if (denom == 0.0) denom = 1e-30;
	z.re = (y.re * x.re + y.im * x.im) / denom;
	z.im = (y.re * x.im - y.im * x.re) / denom;
	return z;
}

Complex sqrt(Complex &x)          /* z = Complex sqrt(x) */
{
	Complex z;
	double h, tmpc;

	if ( x.im == 0.0 ) {      /*argument is pure real*/
		if (x.re >= 0.0 ) z = Complex(sqrt(x.re));
		else z = Complex(0.0,sqrt(-x.re));
	}
      
	else if (x.re == 0.0) {   /*argument is pure imaginary*/
		if ( x.im >= 0.0 ) {
			z.re = sqrt(0.5*x.im);
			z.im = z.re;
		}
		else {
			z.re = sqrt(-0.5*x.im);
			z.im = -z.re;
		}
	}
   
	else  {     /* argument is mixed */
		h = sqrt(x.re*x.re + x.im * x.im);
		tmpc = sqrt(0.5 * (fabs(x.re) + h));
		z.re = tmpc;
		z.im = 0.5 * x.im / tmpc;
		if ( x.re < 0.0 ) {
			z.re = fabs(z.im);
			z.im = ( x.im >= 0.0 ) ? tmpc : -tmpc;
		}
	} 

	return z;
   
}


string Complex::format()
{
	double x, y;
	const int LEN = 64;
	char out[LEN];
	x = (fabs(real())>1e-10? real(): 0.0);
	y = (fabs(imag())>1e-10? imag(): 0.0);
	if (!y) sprintf_s(out,LEN-1,"%g",x);
	else if (!x) sprintf_s(out,LEN-1,"%g j",y);
	else if (y<0.0) sprintf_s(out,LEN-1,"%.4g - %.4g j",x,-y);
	else sprintf_s(out,LEN-1,"%.4g + %.4g j",x,y);
	return string(out);
}

string Complex::phasor()
{
	const int LEN = 64;
	char out[LEN];
	double r,theta;
	r = mag();
	theta = angle();
	if (fabs(theta)<1e-9) theta = 0.0;
	if (!theta) sprintf_s(out,LEN-1,"%g",r);
	else sprintf_s(out,LEN-1,"%g at %g dg",r,theta);
	return string(out);
}